@@ -1,4 +1,3 @@
-import requests
 import asyncio
 import uuid
 import json
@@ -15,15 +14,18 @@ VERSION = "1.6.8"
 MAX_RETRIES = 5
 PUBLIC_AUTH_SERVER = "https://auth.lightwaverf.com/token"
 PUBLIC_API = "https://publicapi.lightwaverf.com/v1/"
+RGB_FLOOR = int("0x1B", 16) #Lightwave app seems to floor RGB values here, let's do the same
 
-class _LWRFMessage:
+#TODO adapt async_connect calls to respond to connection failure
+
+class _LWRFWebsocketMessage:
     _tran_id = 0
     _sender_id = str(uuid.uuid4())
 
     def __init__(self, opclass, operation):
         self._message = {"class": opclass, "operation": operation, "version": 1, "senderId": self._sender_id,
-                         "transactionId": _LWRFMessage._tran_id}
-        _LWRFMessage._tran_id += 1
+                         "transactionId": _LWRFWebsocketMessage._tran_id}
+        _LWRFWebsocketMessage._tran_id += 1
         self._message["direction"] = "request"
         self._message["items"] = []
 
@@ -34,43 +36,56 @@ class _LWRFMessage:
         return json.dumps(self._message)
 
 
-class _LWRFMessageItem:
+class _LWRFWebsocketMessageItem:
     _item_id = 0
 
     def __init__(self, payload=None):
         if payload is None:
             payload = {}
-        self._item = {"itemId": _LWRFMessageItem._item_id}
-        _LWRFMessageItem._item_id += 1
+        self._item = {"itemId": _LWRFWebsocketMessageItem._item_id}
+        _LWRFWebsocketMessageItem._item_id += 1
         self._item["payload"] = payload
 
-class _LWRFFeatureSet:
+
+class LWRFFeatureSet:
 
     def __init__(self):
         self.featureset_id = None
         self.name = None
         self.product_code = None
         self.features = {}
-        self._switchable = False
-        self._dimmable = False
-        self._climate = False
-        self._gen2 = False
 
     def is_switch(self):
-        return self._switchable and not self._dimmable
+        return ('switch' in self.features.keys()) and not ('dimLevel' in self.features.keys())
 
     def is_light(self):
-        return self._dimmable
+        return 'dimLevel' in self.features.keys()
 
     def is_climate(self):
-        return self._climate
+        return 'targetTemperature' in self.features.keys()
+
+    def is_cover(self):
+        return 'threeWayRelay' in self.features.keys()
+
+    def is_energy(self):
+        return ('energy' in self.features.keys()) and ('rssi' in self.features.keys())
+
+    def is_windowsensor(self):
+        return 'windowPosition' in self.features.keys()
 
     def is_gen2(self):
-        return self._gen2
+        return 'identify' in self.features.keys()
+
+    def reports_power(self):
+        return 'power' in self.features.keys()
+
+    def has_led(self):
+        return 'rgbColor' in self.features.keys()
+
 
 class LWLink2:
 
-    def __init__(self, username=None, password=None):
+    def __init__(self, username=None, password=None, auth_method="username", api_token=None, refresh_token=None):
 
         self.featuresets = {}
         self._authtoken = None
@@ -78,9 +93,14 @@ class LWLink2:
         self._username = username
         self._password = password
 
-        self._session = aiohttp.ClientSession()
+        self._auth_method = auth_method
+        self._api_token = api_token
+        self._refresh_token = refresh_token
 
-        #Websocket only variables:
+        self._session = aiohttp.ClientSession()
+        self._token_expiry = None
+
+        # Websocket only variables:
         self._device_id = str(uuid.uuid4())
         self._websocket = None
         self._callback = []
@@ -140,21 +160,12 @@ class LWLink2:
                             and message["operation"] == "event":
                         yield from self.async_get_hierarchy()
                     elif message["direction"] == "notification" and message["operation"] == "event":
-                        #TODO: First call is unecessary, see if second works then refactor the first one out
-                        if "_feature" in message["items"][0]["payload"]:
-                            feature_id = message["items"][0]["payload"]["_feature"]["featureId"]
-                            feature = message["items"][0]["payload"]["_feature"]["featureType"]
-                            value = message["items"][0]["payload"]["value"]
-                            self.get_featureset_by_featureid(feature_id).features[feature][1] = value
-                            _LOGGER.debug("Event with _feature received (%s %s %s), calling callbacks %s", feature_id, feature, value, self._callback)
-                            for func in self._callback:
-                                func()
-                        elif "featureId" in message["items"][0]["payload"]:
+                        if "featureId" in message["items"][0]["payload"]:
                             feature_id = message["items"][0]["payload"]["featureId"]
                             feature = self.get_feature_by_featureid(feature_id)
                             value = message["items"][0]["payload"]["value"]
                             self.get_featureset_by_featureid(feature_id).features[feature][1] = value
-                            _LOGGER.debug("Event with _feature received (%s %s %s), calling callbacks %s", feature_id, feature, value, self._callback)
+                            _LOGGER.debug("Event received (%s %s %s), calling callbacks %s", feature_id, feature, value, self._callback)
                             for func in self._callback:
                                 func()
                         else:
@@ -162,7 +173,7 @@ class LWLink2:
                     else:
                         _LOGGER.warning("Received unhandled message: %s", message)
                 elif mess.type == aiohttp.WSMsgType.CLOSED:
-                # We're not going to get a response, so clear response flag to allow _send_message to unblock
+                    # We're not going to get a response, so clear response flag to allow _send_message to unblock
                     _LOGGER.debug("Websocket closed in message handler")
                     self._waitingforresponse.set()
                     self._transactions = None
@@ -172,15 +183,14 @@ class LWLink2:
             except AttributeError:  # websocket is None if not set up, just wait for a while
                 yield from asyncio.sleep(1)
 
-
     async def async_register_callback(self, callback):
         _LOGGER.debug("Register callback %s", callback)
         self._callback.append(callback)
 
     async def async_get_hierarchy(self):
         _LOGGER.debug("Reading hierarchy")
-        readmess = _LWRFMessage("user", "rootGroups")
-        readitem = _LWRFMessageItem()
+        readmess = _LWRFWebsocketMessage("user", "rootGroups")
+        readitem = _LWRFWebsocketMessageItem()
         readmess.additem(readitem)
         response = await self._async_sendmessage(readmess)
 
@@ -196,8 +206,8 @@ class LWLink2:
     async def _async_read_groups(self):
         self.featuresets = {}
         for groupId in self._group_ids:
-            readmess = _LWRFMessage("group", "read")
-            readitem = _LWRFMessageItem({"groupId": groupId,
+            readmess = _LWRFWebsocketMessage("group", "read")
+            readitem = _LWRFWebsocketMessageItem({"groupId": groupId,
                                          "blocks": True,
                                          "devices": True,
                                          "features": True,
@@ -212,23 +222,17 @@ class LWLink2:
 
                     if z not in self.featuresets:
                         _LOGGER.debug("Creating device {}".format(x))
-                        new_featureset = _LWRFFeatureSet()
+                        new_featureset = LWRFFeatureSet()
                         new_featureset.featureset_id = z
-                        new_featureset.product_code = "Not working" #TODO!
                         new_featureset.name = x["name"]
                         self.featuresets[z] = new_featureset
 
                     _LOGGER.debug("Adding device features {}".format(x))
                     y = self.featuresets[z]
-                    y.features[x["attributes"]["type"]] = [x["featureId"], x["attributes"]["value"]]
-                    if x["attributes"]["type"] == "switch":
-                        y._switchable = True
-                    if x["attributes"]["type"] == "dimLevel":
-                        y._dimmable = True
-                    if x["attributes"]["type"] == "targetTemperature":
-                        y._climate = True
-                    if x["attributes"]["type"] == "identify":
-                        y._gen2 = True
+                    y.features[x["attributes"]["type"]] = [x["featureId"], 0] #Something has changed meaning the server doesn't return values on first call
+
+            for x in list(response["items"][0]["payload"]["devices"].values()):
+                self.get_featureset_by_featureid(x['featureIds'][0]).product_code = x['productCode']
 
     async def async_update_featureset_states(self):
         for dummy, x in self.featuresets.items():
@@ -237,14 +241,14 @@ class LWLink2:
                 x.features[y][1] = value["items"][0]["payload"]["value"]
 
     async def async_write_feature(self, feature_id, value):
-        readmess = _LWRFMessage("feature", "write")
-        readitem = _LWRFMessageItem({"featureId": feature_id, "value": value})
+        readmess = _LWRFWebsocketMessage("feature", "write")
+        readitem = _LWRFWebsocketMessageItem({"featureId": feature_id, "value": value})
         readmess.additem(readitem)
         await self._async_sendmessage(readmess)
 
     async def async_read_feature(self, feature_id):
-        readmess = _LWRFMessage("feature", "read")
-        readitem = _LWRFMessageItem({"featureId": feature_id})
+        readmess = _LWRFWebsocketMessage("feature", "read")
+        readitem = _LWRFWebsocketMessageItem({"featureId": feature_id})
         readmess.additem(readitem)
         return await self._async_sendmessage(readmess)
 
@@ -284,67 +288,114 @@ class LWLink2:
         feature_id = y.features["targetTemperature"][0]
         await self.async_write_feature(feature_id, int(level * 10))
 
+    async def async_cover_open_by_featureset_id(self, featureset_id):
+        y = self.get_featureset_by_id(featureset_id)
+        feature_id = y.features["threeWayRelay"][0]
+        await self.async_write_feature(feature_id, 1)
+
+    async def async_cover_close_by_featureset_id(self, featureset_id):
+        y = self.get_featureset_by_id(featureset_id)
+        feature_id = y.features["threeWayRelay"][0]
+        await self.async_write_feature(feature_id, 2)
+
+    async def async_cover_stop_by_featureset_id(self, featureset_id):
+        y = self.get_featureset_by_id(featureset_id)
+        feature_id = y.features["threeWayRelay"][0]
+        await self.async_write_feature(feature_id, 0)
+
+    async def async_set_led_rgb_by_featureset_id(self, featureset_id, color):
+        red = (color & int("0xFF0000", 16)) >> 16
+        if red != 0:
+            red = min(max(red, RGB_FLOOR), 255)
+        green = (color & int("0xFF00", 16)) >> 8
+        if green != 0:
+            green = min(max(green, RGB_FLOOR), 255)
+        blue = (color & int("0xFF", 16))
+        if blue != 0:
+            blue = min(max(blue , RGB_FLOOR), 255)
+        newcolor = (red << 16) + (green << 8) + blue
+        y = self.get_featureset_by_id(featureset_id)
+        feature_id = y.features["rgbColor"][0]
+        await self.async_write_feature(feature_id, newcolor)
+
     def get_switches(self):
-        temp = []
-        for dummy, x in self.featuresets.items():
-            if x.is_switch():
-                temp.append((x.featureset_id, x.name))
-        return temp
+        return [(x.featureset_id, x.name) for x in self.featuresets.values() if x.is_switch()]
 
     def get_lights(self):
-        temp = []
-        for dummy, x in self.featuresets.items():
-            if x.is_light():
-                temp.append((x.featureset_id, x.name))
-        return temp
+        return [(x.featureset_id, x.name) for x in self.featuresets.values() if x.is_light()]
 
     def get_climates(self):
-        temp = []
-        for dummy, x in self.featuresets.items():
-            if x.is_climate():
-                temp.append((x.featureset_id, x.name))
-        return temp
+        return [(x.featureset_id, x.name) for x in self.featuresets.values() if x.is_climate()]
+
+    def get_covers(self):
+        return [(x.featureset_id, x.name) for x in self.featuresets.values() if x.is_cover()]
+
+    def get_energy(self):
+        return [(x.featureset_id, x.name) for x in self.featuresets.values() if x.is_energy()]
+
+    def get_windowsensors(self):
+        return [(x.featureset_id, x.name) for x in self.featuresets.values() if x.is_windowsensor()]
 
     #########################################################
     # Connection
     #########################################################
 
-    async def async_connect(self, tries=0):
+    async def async_connect(self, max_tries=5, _retry=0):
         try:
-            if not self._websocket or self._websocket.closed:
-                self._websocket = await self._session.ws_connect(TRANS_SERVER)
-            return await self._authenticate()
+            return await self._connect_to_server()
         except Exception as exp:
-            retry_delay = min(2 ** (tries + 1), 120)
-            _LOGGER.warning("Cannot connect (exception '{}'). Waiting {} seconds".format(exp, retry_delay))
-            await asyncio.sleep(retry_delay)
-            return await self.async_connect(tries + 1)
+            if (max_tries == 0) or (_retry < max_tries):
+                retry_delay = min(2 ** (_retry + 1), 120)
+                _LOGGER.warning("Cannot connect (exception '{}'). Waiting {} seconds to retry".format(exp, retry_delay))
+                await asyncio.sleep(retry_delay)
+                return await self.async_connect(max_tries, _retry + 1)
+            else:
+                _LOGGER.warning("Cannot connect, max_tries exceeded, aborting")
+                return False
 
-    async def _authenticate(self):
+    async def _connect_to_server(self):
+        if (not self._websocket) or self._websocket.closed:
+            _LOGGER.debug("Connecting to websocket")
+            self._websocket = await self._session.ws_connect(TRANS_SERVER)
+        return await self._authenticate_websocket()
+
+    async def _authenticate_websocket(self):
         if not self._authtoken:
             await self._get_access_token()
         if self._authtoken:
-            authmess = _LWRFMessage("user", "authenticate")
-            authpayload = _LWRFMessageItem({"token": self._authtoken, "clientDeviceId": self._device_id})
+            authmess = _LWRFWebsocketMessage("user", "authenticate")
+            authpayload = _LWRFWebsocketMessageItem({"token": self._authtoken, "clientDeviceId": self._device_id})
             authmess.additem(authpayload)
             response = await self._async_sendmessage(authmess)
             if not response["items"][0]["success"]:
                 if response["items"][0]["error"]["code"] == "200":
-                    #"Channel is already authenticated" - Do nothing
+                    # "Channel is already authenticated" - Do nothing
                     pass
                 elif response["items"][0]["error"]["code"] == 405:
-                    #"Access denied" - bogus token, let's reauthenticate
-                    #Lightwave seems to return a string for 200 but an int for 405!
+                    # "Access denied" - bogus token, let's reauthenticate
+                    # Lightwave seems to return a string for 200 but an int for 405!
                     _LOGGER.debug("Authentication token rejected, regenerating and reauthenticating")
                     self._authtoken = None
-                    await self._authenticate()
+                    await self._authenticate_websocket()
+                elif response["items"][0]["error"]["message"] == "user-msgs: Token not valid/expired.":
+                    _LOGGER.debug("Authentication token expired, regenerating and reauthenticating")
+                    self._authtoken = None
+                    await self._authenticate_websocket()
                 else:
-                    _LOGGER.debug("Unhandled authentication error {}".format(response["items"][0]["error"]["message"]))
+                    _LOGGER.warning("Unhandled authentication error {}".format(response["items"][0]["error"]["message"]))
             return response
         else:
             return None
 
     async def _get_access_token(self):
+        if self._auth_method == "username":
+            await self._get_access_token_username()
+        elif self._auth_method == "api":
+            await self._get_access_token_api()
+        else:
+            raise ValueError("auth_method must be 'username' or 'api'")
+
+    async def _get_access_token_username(self):
         _LOGGER.debug("Requesting authentication token (using username and password)")
         authentication = {"email": self._username, "password": self._password, "version": VERSION}
         async with self._session.post(AUTH_SERVER, headers={"x-lwrf-appid": "ios-01"}, json=authentication) as req:
@@ -355,6 +406,22 @@ class LWLink2:
                 _LOGGER.warning("No authentication token (status_code '{}').".format(req.status))
                 raise ConnectionError("No authentication token: {}".format(await req.text))
 
+    # TODO check for token expiry
+    async def _get_access_token_api(self):
+        _LOGGER.debug("Requesting authentication token (using API key and refresh token)")
+        authentication = {"grant_type": "refresh_token", "refresh_token": self._refresh_token}
+        async with self._session.post(PUBLIC_AUTH_SERVER,
+                            headers={"authorization": "basic " + self._api_token},
+                            json=authentication) as req:
+            _LOGGER.debug("Received response: {}".format(await req.text))
+            if req.status == 200:
+                self._authtoken = await req.json()["access_token"]
+                self._refresh_token = await req.json()["refresh_token"]
+                self._token_expiry = datetime.datetime.now() + datetime.timedelta(seconds=await req.json()["expires_in"])
+            else:
+                _LOGGER.warning("No authentication token (status_code '{}').".format(req.status))
+                raise ConnectionError("No authentication token: {}".format(await req.text))
+
     #########################################################
     # Convenience methods for non-async calls
     #########################################################
@@ -386,9 +453,19 @@ class LWLink2:
     def set_temperature_by_featureset_id(self, featureset_id, level):
         return asyncio.get_event_loop().run_until_complete(self.async_set_temperature_by_featureset_id(featureset_id, level))
 
+    def cover_open_by_featureset_id(self, featureset_id):
+        return asyncio.get_event_loop().run_until_complete(self.async_cover_open_by_featureset_id(featureset_id))
+
+    def cover_close_by_featureset_id(self, featureset_id):
+        return asyncio.get_event_loop().run_until_complete(self.async_cover_close_by_featureset_id(featureset_id))
+
+    def cover_stop_by_featureset_id(self, featureset_id):
+        return asyncio.get_event_loop().run_until_complete(self.async_cover_stop_by_featureset_id(featureset_id))
+
     def connect(self):
         return asyncio.get_event_loop().run_until_complete(self.async_connect())
 
+
 class LWLink2Public(LWLink2):
 
     def __init__(self, username=None, password=None, auth_method="username", api_token=None, refresh_token=None):
@@ -398,6 +475,7 @@ class LWLink2Public(LWLink2):
 
         self._username = username
         self._password = password
+
         self._auth_method = auth_method
         self._api_token = api_token
         self._refresh_token = refresh_token
@@ -405,21 +483,42 @@ class LWLink2Public(LWLink2):
         self._session = aiohttp.ClientSession()
         self._token_expiry = None
 
-    #TODO add retries/error checking
-    #TODO make this async!
+    # TODO add retries/error checking to public API requests
     async def _async_getrequest(self, endpoint, _retry=1):
         _LOGGER.debug("Sending API GET request to {}".format(endpoint))
-        req = requests.get(PUBLIC_API + endpoint,
-                           headers={"authorization": "bearer " + self._authtoken})
-        _LOGGER.debug("Received API response {}".format(req.json()))
-        return req.json()
+        async with self._session.get(PUBLIC_API + endpoint,
+                                     headers= {"authorization": "bearer " + self._authtoken}
+                                      ) as req:
+            _LOGGER.debug("Received API response {} {} {}".format(req.status, await req.text(), await req.json()))
+            return await req.json()
 
     async def _async_postrequest(self, endpoint, body="", _retry=1):
         _LOGGER.debug("Sending API POST request to {}: {}".format(endpoint, body))
-        req = requests.post(PUBLIC_API + endpoint,
-                            headers={"authorization": "bearer " + self._authtoken}, json=body)
-        _LOGGER.debug("Received API response {}".format(req.json()))
-        return req.json()
+        async with self._session.post(PUBLIC_API + endpoint,
+                                      headers= {"authorization": "bearer " + self._authtoken},
+                                      json=body) as req:
+            _LOGGER.debug("Received API response {} {} {}".format(req.status, await req.text(), await req.json()))
+            if not(req.status == 401 and (await req.json())['message'] == 'Unauthorized'):
+                return await req.json()
+        try:
+            _LOGGER.debug("POST failed due to unauthorized connection, retrying connect")
+            await self.async_connect()
+            async with self._session.post(PUBLIC_API + endpoint,
+                                          headers={
+                                              "authorization": "bearer " + self._authtoken},
+                                          json=body) as req:
+                _LOGGER.debug("Received API response {} {} {}".format(req.status, await req.text(), await req.json()))
+                return await req.json()
+        except:
+            return False
+
+    async def _async_deleterequest(self, endpoint, _retry=1):
+        _LOGGER.debug("Sending API DELETE request to {}".format(endpoint))
+        async with self._session.delete(PUBLIC_API + endpoint,
+                                     headers= {"authorization": "bearer " + self._authtoken}
+                                      ) as req:
+            _LOGGER.debug("Received API response {} {} {}".format(req.status, await req.text(), await req.json()))
+            return await req.json()
 
     async def async_get_hierarchy(self):
 
@@ -431,7 +530,7 @@ class LWLink2Public(LWLink2):
             for x in response["devices"]:
                 for y in x["featureSets"]:
                     _LOGGER.debug("Creating device {}".format(x))
-                    new_featureset = _LWRFFeatureSet()
+                    new_featureset = LWRFFeatureSet()
                     new_featureset.featureset_id = y["featureSetId"]
                     new_featureset.product_code = x["productCode"]
                     new_featureset.name = x["name"]
@@ -439,29 +538,47 @@ class LWLink2Public(LWLink2):
                     for z in y["features"]:
                         _LOGGER.debug("Adding device features {}".format(z))
                         new_featureset.features[z["type"]] = [z["featureId"], None]
-                        if z["type"] == "switch":
-                            new_featureset._switchable = True
-                        if z["type"] == "dimLevel":
-                            new_featureset._dimmable = True
-                        if z["type"] == "targetTemperature":
-                            new_featureset._climate = True
-                        if z["type"] == "identify":
-                            new_featureset._gen2 = True
+
                     self.featuresets[y["featureSetId"]] = new_featureset
 
         await self.async_update_featureset_states()
 
-    # TODO ######################################
     async def async_register_callback(self, callback):
         pass
 
+    async def async_register_webhook(self, url, feature_id, ref, overwrite = False):
+        if overwrite:
+            req = await self._async_deleterequest("events/" + ref)
+        payload = {"events": [{"type": "feature", "id": feature_id}],
+                    "url": url,
+                    "ref": ref}
+        req = await self._async_postrequest("events", payload)
+        #TODO: test for req = 200
+
+    async def delete_all_webhooks(self):
+        webhooks = await self._async_getrequest("events")
+        for wh in webhooks:
+            await self._async_deleterequest("events/" + wh["id"])
+
+    async def async_delete_webhook(self, ref):
+        req = await self._async_deleterequest("events/" + ref)
+        #TODO: test for req = 200
+
+    def process_webhook_received(self, body):
+
+        featureid = body['triggerEvent']['id']
+        featureset = self.get_featureset_by_featureid(featureid)
+        featurename = self.get_feature_by_featureid(featureid)
+        value = body['payload']['value']
+        featureset.features[featurename][1] = value
+
     async def async_update_featureset_states(self):
         feature_list = []
 
         for dummy, x in self.featuresets.items():
             for y in x.features:
-                feature_list.append({"featureId":x.features[y][0]})
-        body = {"features":feature_list}
+                feature_list.append({"featureId": x.features[y][0]})
+        body = {"features": feature_list}
         req = await self._async_postrequest("features/read", body)
 
         for featuresetid in self.featuresets:
@@ -481,50 +598,9 @@ class LWLink2Public(LWLink2):
     # Connection
     #########################################################
 
-    async def async_connect(self, tries=0):
-        try:
-            self._get_access_token()
-        except Exception as exp:
-            retry_delay = min(2 ** (tries + 1), 120)
-            _LOGGER.warning("Cannot connect (exception '{}'). Waiting {} seconds".format(exp, retry_delay))
-            await asyncio.sleep(retry_delay)
-            return await self.async_connect(tries + 1)
-    #TODO distinguish failure on no token and don't retry
+    async def _connect_to_server(self):
+            await self._get_access_token()
+            return True
 
-    def _get_access_token(self):
-        if self._auth_method == "username":
-            self._get_access_token_username()
-        elif self._auth_method == "api":
-            self._get_access_token_api()
-        else:
-            raise ValueError("auth_method must be 'username' or 'api'")
 
-    def _get_access_token_username(self):
-        _LOGGER.debug("Requesting authentication token (using username and password)")
-        authentication = {"email": self._username, "password": self._password, "version": VERSION}
-        req = requests.post(AUTH_SERVER,
-                            headers={"x-lwrf-appid": "ios-01"},
-                            json=authentication)
-        _LOGGER.debug("Received response: {}".format(req.json()))
-        if req.status_code == 200:
-            token = req.json()["tokens"]["access_token"]
-            self._authtoken = token
-        else:
-            _LOGGER.warning("No authentication token (status_code '{}').".format(req.status_code))
-            raise ConnectionError("No authentication token: {}".format(req.text))
 
-    #TODO check for token expiry
-    def _get_access_token_api(self):
-        _LOGGER.debug("Requesting authentication token (using API key and refresh token)")
-        authentication = {"grant_type": "refresh_token", "refresh_token": self._refresh_token}
-        req = requests.post(PUBLIC_AUTH_SERVER,
-                            headers={"authorization": "basic " + self._api_token},
-                            json=authentication)
-        _LOGGER.debug("Received response: {}".format(req))
-        if req.status_code == 200:
-            self._authtoken = req.json()["access_token"]
-            self._refresh_token = req.json()["refresh_token"]
-            self._token_expiry = datetime.datetime.now() + datetime.timedelta(seconds=req.json()["expires_in"])
-        else:
-            _LOGGER.warning("No authentication token (status_code '{}').".format(req.status_code))
-            raise ConnectionError("No authentication token: {}".format(req.text))
